@@ -16,26 +16,26 @@ from ast import parse
 from ast import NodeVisitor
 
 
-class JSObject(dict):
-
-    def length(self):
-        return len(self)
-
-
-class JSArray(list):
-
-    def push(self, value):
-        self.append(value)
-
-    def length(self):
-        return len(self)
-
-
 class JSGenerator(NodeVisitor):
 
     def visit_Module(self, node):
         return '\n'.join(map(self.visit, node.body))
 
+    def visit_TryExcept(self, node):
+        out = 'try {\n'
+        out += '\n'.join(map(self.visit, node.body))
+        out += '\n}\n'
+        out += 'catch(__exception__) {\n'
+        out += '\n'.join(map(self.visit, node.handlers))
+        out += '\n}\n'
+        return out
+
+    def visit_Raise(self, node):
+        return 'throw %s;' % self.visit(node.type)
+
+    def visit_ExceptHandler(self, node):
+        return '\n'.join(map(self.visit, node.body))
+
     def visit_FunctionDef(self, node):
         args = self.visit(node.args)
         buffer = 'var %s = function(%s) {\n' % (
@@ -66,6 +66,10 @@ class JSGenerator(NodeVisitor):
     def visit_Name(self, node):
         if node.id == 'None':
             return 'undefined'
+        if node.id == 'True':
+            return 'true'
+        if node.id == 'False':
+            return 'false'
         return node.id
 
     def visit_Attribute(self, node):
@@ -75,7 +79,7 @@ class JSGenerator(NodeVisitor):
 
     def visit_Print(self, node):
         args = [self.visit(e) for e in node.values]
-        s = 'console.log(%s)' % ' + '.join(args)
+        s = 'console.log(%s);' % ', '.join(args)
         return s
 
     def visit_keyword(self, node):
@@ -93,6 +97,10 @@ class JSGenerator(NodeVisitor):
             else:
                 out = ''
             return '{%s}' % out
+        if name == 'var':
+            args = map(self.visit, node.args)
+            out = ', '.join(args)
+            return 'var %s' % out
         if name == 'JSArray':
             if node.args:
                 args = map(self.visit, node.args)
@@ -144,11 +152,15 @@ class JSGenerator(NodeVisitor):
     def visit_Assign(self, node):
         target = self.visit(node.targets[0])  # XXX: support only one target
         value = self.visit(node.value)
-        code = '%s = %s' % (target, value)
+        code = '%s = %s;' % (target, value)
         return code
 
     def visit_Expr(self, node):
-        return self.visit(node.value) + ';'
+        # XXX: this is UGLY
+        s = self.visit(node.value)
+        if not s.endswith(';'):
+            s += ';'
+        return s
 
     def visit_Return(self, node):
         if node.value:
@@ -173,10 +185,16 @@ class JSGenerator(NodeVisitor):
         comparator = self.visit(node.comparators[0])
         return '%s %s %s' % (left, ops, comparator)
 
+    def visit_Not(self, node):
+        return '!'
+
+    def visit_UnaryOp(self, node):
+        return self.visit(node.op) + self.visit(node.operand)
+
     def visit_If(self, node):
         test = self.visit(node.test)
-        body = '\n'.join(map(self.visit, node.body))
-        orelse = '\n'.join(map(self.visit, node.orelse))
+        body = '\n'.join(map(self.visit, node.body)) + '\n'
+        orelse = '\n'.join(map(self.visit, node.orelse)) + '\n'
         if orelse:
             return 'if(%s) {\n%s}\nelse {\n%s}\n' % (test, body, orelse)
         return 'if(%s) {\n%s}\n' % (test, body)
@@ -187,12 +205,12 @@ class JSGenerator(NodeVisitor):
         # iter is the python iterator
         init_iter = 'var iter = %s;\n' % iter
         # backup iterator and affect value of the next element to the target
-        pre = 'var backup = %s\ni = iter[%s];\n' % (target, target)
+        pre = 'var backup = %s;\ni = iter[%s];\n' % (target, target)
         # replace the replace target with the javascript iterator
-        post = 'i = backup\n'
+        post = 'i = backup;\n'
         body = '\n'.join(map(self.visit, node.body)) + '\n'
         body = pre + body + post
-        for_block = init_iter + 'for (%s=0; %s < iter.length; %s++) {\n%s}\n' % (target, target, target, body)
+        for_block = init_iter + 'for (var %s=0; %s < iter.length; %s++) {\n%s}\n' % (target, target, target, body)
         return for_block
 
 
