@@ -1,22 +1,25 @@
-def create_array():
-    """Used to fix a bug/feature of Javascript where new Array(number)
-    created a array with number of undefined elements which is not
-    what we want"""
-    JS('array = new Array()')
-    for i in range(arguments.length):
-        JS('array.push(arguments[i])')
-    return array
-
-
-def range(num):
+def jsrange(num):
     """Emulates Python's range function"""
-    i = 1
-    r = JS('[0]')
+    JS('var i')
+    JS('var r')
+    i = 0
+    r = JS('[]')
     while i < num:
         r.push(i)
         i = i + 1
     return r
 
+
+def create_array():
+    """Used to fix a bug/feature of Javascript where new Array(number)
+    created a array with number of undefined elements which is not
+    what we want"""
+    JS('var array = new Array()')
+    for i in jsrange(arguments.length):
+        JS('array.push(arguments[i])')
+    return array
+
+
 def adapt_arguments(handler):
     """Useful to transform callback arguments to positional arguments"""
     def func():
@@ -24,29 +27,26 @@ def adapt_arguments(handler):
     return func
 
 
-def create_object():
-    """Create a PythonScript object"""
-    object = JSObject()
-    object.__class__ = klass
-    object.__dict__ = JSObject()
-
-    init = get_attribute(object, '__init__')
-    if init:
-        init.apply(None, arguments)
-    return object
-
-
 def create_class(class_name, parents, attrs):
     """Create a PythonScript class"""
+    JS('var klass')
     klass = JSObject()
     klass.bases = parents
     klass.__name__ = class_name
     klass.__dict__ = attrs
 
     def __call__():
-        args = arguments.___insert(0, klass)
-        return create_object.apply(None, args)
-    klass.__call__ = create_object
+        """Create a PythonScript object"""
+        JS('var object')
+        object = JSObject()
+        object.__class__ = klass
+        object.__dict__ = JSObject()
+        JS('var init')
+        init = get_attribute(object, '__init__')
+        if init:
+            init.apply(None, arguments)
+        return object
+    klass.__call__ = __call__
     return klass
 
 
@@ -59,30 +59,44 @@ def get_attribute(object, attribute):
     if attribute == '__call__':
         if JS("{}.toString.call(object) === '[object Function]'"):
             return object
-    attr = JS('object[attribute]')
+    JS('var attr = object[attribute]')
     if attr:
         return attr
-    __dict__ = object.__dict__
+    JS('var __dict__ = object.__dict__')
     if __dict__:
         attr = JS('__dict__[attribute]')
-        if attr:
+        if attr != None:
             return attr
-    __class__ = object.__class__
+    JS('var __class__ = object.__class__')
     if __class__:
-        __dict__ = __class__.__dict__
+        JS('var __dict__ = __class__.__dict__')
         attr = JS('__dict__[attribute]')
         if attr:
             if JS("{}.toString.call(attr) === '[object Function]'"):
                 def method():
-                    JS('arguments[0]').splice(0, 0, object)
-                    return attr.apply(None, arguments)
+                    JS('var args = arguments')
+                    if(args.length>0):
+                        JS('args[0]').splice(0, 0, object)
+                    else:
+                        args = JSArray(object)
+                    return attr.apply(None, args)
                 return method
             return attr
-        bases = __class__.bases
-        for i in range(bases.length):
-            base = JS('bases[i]')
-            attr = get_attribute(base, attribute)
+        JS('var bases = __class__.bases')
+        for i in jsrange(bases.length):
+            JS('var base = bases[i]')
+            JS('var attr = get_attribute(base, attribute)')
             if attr:
+                if JS("{}.toString.call(attr) === '[object Function]'"):
+                    def method():
+                        JS('var args = arguments')
+                        if(args.length>0):
+                            JS('args[0]').splice(0, 0, object)
+                        else:
+                            args = JSArray(object)
+                        return attr.apply(None, args)
+                    return method
+
                 return attr
     return None
 
@@ -100,19 +114,18 @@ def get_arguments(signature, args, kwargs):
     This will set default keyword arguments and retrieve positional arguments
     in kwargs if their called as such"""
     out = JSObject()
-    if parameters.args.length:
-        argslength = parameters.args.length
+    if signature.args.length:
+        argslength = signature.args.length
     else:
         argslength = 0
-    kwargslength = JS('Object.keys(parameters.kwargs).length')
+    kwargslength = JS('Object.keys(signature.kwargs).length')
     j = 0
-    for i in range(argslength):
-        arg = JS('parameters.args[j]')
+    for i in jsrange(argslength):
+        arg = JS('signature.args[j]')
         if kwargs:
             kwarg = JS('kwargs[arg]')
             if kwarg:
                 JS('out[arg] = kwarg')
-                JS('delete kwargs[arg]')
             else:
                 JS('out[arg] = args[j]')
                 j = j + 1
@@ -120,8 +133,8 @@ def get_arguments(signature, args, kwargs):
             JS('out[arg] = args[j]')
             j = j + 1
     args = args.slice(j)
-    if parameters.vararg:
-        JS("out[parameters.vararg] = args")
-    if parameters.varkwarg:
-        JS("out[parameters.varkwarg] = kwargs")
+    if signature.vararg:
+        JS("out[signature.vararg] = args")
+    if signature.varkwarg:
+        JS("out[signature.varkwarg] = kwargs")
     return out
